package	jp.co.sds_corp.streaming;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.font.LineMetrics;
import java.util.HashMap;
import java.util.Iterator;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.ratemixer.fx.FxRateKey;
import jp.co.sds_corp.transfer.IReceiveListener;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.ServiceLoader;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.pooh.simple.SimplePanel;
import jp.co.sds_corp.util.window.ScalableFontButton;

public class StreamingPanel extends SimplePanel
	implements IReceiveListener
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	ServiceLoader	recv;
	String		ccy1, ccy2, term, pairs;
	int			nPrec;
	MasterCache	master;
	CcyPair	pair;
	FxRateKey	key;
	ScalableFontButton	BTN_T;
	RateButton	BTN_C;
	RateButton	BTN_L;
	RateButton	BTN_R;

	public StreamingPanel() {
		super();
	}

	public boolean createPane() {
		setLayout( null );
		BTN_T = new ScalableFontButton( "" );
		BTN_C = new RateButton( "..", RateButton.INT );
		BTN_L = new RateButton( "..", RateButton.BID );
		BTN_R = new RateButton( "..", RateButton.ASK );

		BTN_T.setMargin( new Insets( 0, 0, 0, 0 ));
		BTN_C.setMargin( new Insets( 0, 0, 0, 0 ));
		BTN_L.setMargin( new Insets( 0, 0, 0, 0 ));
		BTN_R.setMargin( new Insets( 0, 0, 0, 0 ));

		BTN_T.setFocusPainted(false);

		add( BTN_T );
		add( BTN_C );
		add( BTN_L );
		add( BTN_R );
		return true;
	}

	public void initPanel( String ccy1, String ccy2, String term, String prec ) {
		this.ccy1 = ccy1;
		this.ccy2 = ccy2;
		this.term = term;
		pairs = ccy1 + ccy2 + term;
		this.nPrec = Integer.valueOf( prec ).intValue();
		master = MasterCache.getInstance();

		BTN_T.setText( ccy1 + "/" + ccy2 + " " + term );
		BTN_C.setPair( ccy1+ccy2+term, nPrec );
		BTN_L.setPair( ccy1+ccy2+term, nPrec );
		BTN_R.setPair( ccy1+ccy2+term, nPrec );

		try {
			pair = master.getCcyPair( ccy1, ccy2 );
			key = new FxRateKey( pair, term );
			recv = ServiceLoader.getInstance();

			FxRate rate = null;
			Iterator itr = recv.getCurrent( key ).iterator();
			while ( itr.hasNext()) {
				rate = (FxRate )itr.next();
			}
			if ( rate != null ) {
				BTN_C.setRate( rate.getBid(), rate.getBid() );
				BTN_L.setRate( rate.getBid(), rate.getBid() );
				BTN_R.setRate( rate.getAsk(), rate.getAsk() );
			}

			recv.addReceiveListener( key, this );
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

	int nW = -1, nH = -1;
	public void reshape( int x, int y, int w, int h ) {
		super.reshape( x, y, w, h );
		if ( nW != w || nH != h ) {
			int	nT = h / 5;
			if ( nT < 24 ) {
				nT = 24;
			}
			BTN_T.setBounds( 0, 0, w, nT );
			BTN_C.setBounds( 0, nT, w/5, h - nT );
			BTN_L.setBounds( w/5, nT, w/5*2, h - nT );
			BTN_R.setBounds( w/5*3, nT, w/5*2, h - nT );
			LineMetrics lmc = BTN_C.getLineMetrics();
			LineMetrics lmr = BTN_R.getLineMetrics();
			if ( lmc == null || lmr == null ) {
				return;
			}
			BTN_C.setMargin( new Insets( 0, 0,
					(int )(lmr.getDescent()-lmc.getDescent()), 0 ));
			nW = w;
			nH = h;
		}
	}

	HashMap mapRate = new HashMap();
	boolean	bInit = false;
	public void received( ReceiveEvent e ) {
		try {
			FxRate newR = (FxRate )e.getData();
			FxRate oldR = (FxRate )mapRate.get( newR.getKey() );
			if ( oldR == null ) {
				oldR = new FxRate( (FxRateKey )newR.getKey() );
			}
			logger.debug( this, "changed:" + oldR + "->" + newR );
			if ( !newR.getTerm().equals( term )) {
				return;
			}
			BTN_C.setRate( oldR.getBid(), newR.getBid() );
			BTN_L.setRate( oldR.getBid(), newR.getBid() );
			BTN_R.setRate( oldR.getAsk(), newR.getAsk() );
			if ( !bInit ) {
				nW = -1;
				nH = -1;
				Rectangle rect = getBounds();
				reshape( (int )rect.getX(), (int )rect.getY(),
							(int )rect.getWidth(), (int )rect.getHeight() );
				bInit = true;
			}
			mapRate.put( newR.getKey(), newR );
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

	public void statusChanged( StatusChangeEvent e ) {
	}

	public String getCcy1() {
		return ccy1;
	}
	public String getCcy2() {
		return ccy2;
	}
	public String getTerm() {
		return term;
	}
	public int getPrec() {
		return nPrec;
	}
	public void setCcy1( String var ) {
		ccy1 = var;
	}
	public void setCcy2( String var ) {
		ccy2 = var;
	}
	public void setTerm( String var ) {
		term = var;
	}
	public void setPrec( int var ) {
		nPrec = var;
	}

	public Component getPopupComponent() {
		return BTN_R;
	}

	public String getPairs() {
		return pairs;
	}

	public void closed() {
		try {
			recv.removeReceiveListener( key, this );
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
